@@ -1,3 +1,6 @@
+#[macro_use]
+extern crate derive_is_enum_variant;
+
 mod errors;
 mod lex;
 mod parse;
@@ -6,7 +9,7 @@ mod scope;
 mod shapes;
 mod signature;
 
-pub use lex::{group, lex, LiteBlock, LiteCommand, LiteGroup, LitePipeline};
+pub use lex::{block, lex, LiteBlock, LiteCommand, LiteGroup, LitePipeline};
 pub use parse::{classify_block, garbage, parse, parse_full_column_path, parse_math_expression};
 pub use path::expand_ndots;
 pub use scope::ParserScope;
